@@ -180,23 +180,34 @@ def merge_related_runs(runs, quiet=False):
             [runs.pop(key) for key in keys]
 
 
-def calc_TOF(t_pulse, t_signal):
+def calc_TOF(t_pulse, t_signal, mode='default'):
     """Calculate TOF from pulse and signal time arrays."""
-    tof = []
-    idx = 1
-    for t in t_signal:
-        idx = bisect_left(t_pulse, t)
-        if idx == len(t_pulse):
-            t_0 = t_pulse.iloc[-1]
-        elif idx == 0:
-            t_0 = 0
-            print("\nWARNING: count detected before first pulse!")
-        else:
-            t_0 = t_pulse[idx - 1]
-        tof.append((t - t_0) / 1e6)  # convert to ps to us
+    if mode == 'pd':
+        idx = t_pulse.searchsorted(t_signal, side='left')
+        idx = [i-1 if i != 0 else 0 for i in idx]
+        t_pulse_tof = t_pulse.iloc[idx]
+        tof = (t_signal.values - t_pulse_tof.values)/1e6
+    else:
+        tof = []
+        idx = 1
+        for t in t_signal:
+            idx = bisect_left(t_pulse, t)
+            if idx == len(t_pulse):
+                t_0 = t_pulse.iloc[-1]
+            elif idx == 0:
+                t_0 = 0
+                print("\nWARNING: count detected before first pulse!")
+            else:
+                t_0 = t_pulse[idx - 1]
+            tof.append((t - t_0) / 1e6)  # convert to ps to us
     return tof
 
 
+def calc_TOF_pd(t_pulse, t_signal):
+    """Calculate TOF from pulse and signal time arrays using pandas."""
+    t_pulse.searchsorted(t_signal, side='left')
+
+
 def calc_trans(counts_in, counts_out, t_meas_in, t_meas_out):
     """Calculate transmission and propagate error."""
     vals_trans = [
